/*
* Copyright 2010-2011 Research In Motion Limited.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package blackberry.io.dir;

import net.rim.device.api.io.file.FileIOException;

import blackberry.core.FunctionSignature;
import blackberry.core.ScriptableFunctionBase;
import blackberry.io.FileConnectionWrapper;

/**
 * JavaScript funciton class - retrieves the parent directory given the specified path
 */
public final class GetParentDirectoryFunction extends ScriptableFunctionBase {

    public static final String NAME = "getParentDirectory";

    /**
     * Determine the path to the parent directory .
     * 
     * @param args
     *            args[0]: the complete URL path of the file/direcoty.
     * @return the complete URL path of the parent directory.
     */

    public Object execute( Object thiz, Object[] args ) throws Exception {

        String parentDir = null;
        FileConnectionWrapper fConnWrap = null;

        try {
            fConnWrap = new FileConnectionWrapper( args[ 0 ].toString() );

            if( !fConnWrap.exists() ) {
                throw new FileIOException( FileIOException.FILENAME_NOT_FOUND );
            }

            parentDir = "file://" + fConnWrap.getFileConnection().getPath();
        } finally {
            if( fConnWrap != null ) {
                fConnWrap.close();
            }
        }

        return parentDir;
    }

    /**
     * @see blackberry.core.ScriptableFunctionBase#getFunctionSignatures
     */
    protected FunctionSignature[] getFunctionSignatures() {
        FunctionSignature fs = new FunctionSignature( 1 );
        fs.addParam( String.class, true );
        return new FunctionSignature[] { fs };
    }
}
